import panel as pn

LOGO = "https://avatars.githubusercontent.com/u/57169982?s=200&v=4"
ACCENT = "#db2777"
MERMAID_JS_URL = "https://mermaid.js.org/"
PANEL_MERMAID_URL = "https://github.com/awesome-panel/panel-mermaid"
EXAMPLES = {
    "Default": """\
graph LR
    A[Hello] --> B[fa:fa-chart-simple Panel] --> E[World]
    A-->C(fa:fa-diagram-project Mermaid) --> E ;
""",
    "Architecture": """\
architecture-beta
    group api(cloud)[API]

    service db(database)[Database] in api
    service disk1(disk)[Storage] in api
    service disk2(disk)[Storage] in api
    service server(server)[Server] in api

    db:L -- R:server
    disk1:T -- B:server
    disk2:T -- B:db
""",
}

BOOTSTRAP_CSS_URL = (
    "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.6.0/css/all.min.css"
)


def get_json_theme():
    if pn.config.theme == "dark":
        return "dark"
    return "light"


def get_code_editor_theme():
    if pn.config.theme == "dark":
        return "chaos"
    return "crimson_editor"



from panel.custom import Child, JSComponent

CSS = """
.split {
    display: flex;
    flex-direction: row;
    height: 100%;
    width: 100%;
}

.gutter {
    background-color: var(--light-border-subtle);
    background-repeat: no-repeat;
    background-position: 50%;
}

.gutter.gutter-horizontal {
    background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAeCAYAAADkftS9AAAAIklEQVQoU2M4c+bMfxAGAgYYmwGrIIiDjrELjpo5aiZeMwF+yNnOs5KSvgAAAABJRU5ErkJggg==');
    cursor: col-resize;
}
"""


# Inspired by https://panel.holoviz.org/how_to/custom_components/esm/custom_layout.html
class Splitter(JSComponent):

    left = Child()
    right = Child()

    _esm = """
    import Split from 'https://esm.sh/split.js@1.6.5'

    export function render({ model }) {
      const splitDiv = document.createElement('div');
      splitDiv.className = 'split';

      const split0 = document.createElement('div');
      splitDiv.appendChild(split0);

      const split1 = document.createElement('div');
      splitDiv.appendChild(split1);

      const split = Split([split0, split1])

      model.on('remove', () => split.destroy())

      split0.append(model.get_child("left"))
      split1.append(model.get_child("right"))
      return splitDiv
    }"""

    _stylesheets = [CSS]



import panel as pn
import param
from panel.widgets import WidgetBase

NO_EXAMPLE = ""


# Inspired by https://www.gradio.app/docs/gradio/examples
class ExamplePicker(pn.viewable.Viewer, WidgetBase):
    value = param.Parameter()

    options = param.ClassSelector(class_=(list, dict))
    components = param.Parameter(allow_refs=False)

    def __init__(self, **kwargs):
        if "value" in kwargs and "options" in kwargs:
            value = kwargs["value"]
            options = kwargs["options"]
            if value in options and value not in options.values():
                kwargs["value"] = options[value]

        super().__init__(**kwargs)
        self._selector = pn.widgets.Select(
            value=NO_EXAMPLE, options=self._options, margin=(-10, 10, 10, 10)
        )
        pn.bind(self._update_value, self._selector.param.value, watch=True)()

        if "value" in kwargs:
            self.param.trigger("value")

        self._layout = pn.Column(
            pn.pane.Markdown("**Pick an Example 👇**", margin=(0, 10)), self._selector
        )

    @param.depends("options")
    def _options(self):
        options = self.options
        if isinstance(options, dict):
            return {NO_EXAMPLE: NO_EXAMPLE} | options
        return [NO_EXAMPLE] + self.options

    def _update_value(self, value):
        if value == NO_EXAMPLE:
            return

        if self.value != value:
            self.value = value
        else:
            self.param.trigger("value")

        self._selector.value = None

    @param.depends("value", watch=True)
    def _update_components(self):
        components = self.components
        if not components:
            return
        value = self.value
        if isinstance(components, list):
            raise NotImplementedError
        if hasattr(components, "object"):
            components.object = value
        elif hasattr(components, "value"):
            components.value = value
        else:
            raise ValueError(
                "Error. Component does not have an object or value attribute."
            )

    def __panel__(self):
        return self._layout




import panel as pn
import param
from panel.custom import JSComponent
from panel.widgets import WidgetBase

VERSION = 11

THEME_MAP = {"default": "default", "dark": "dark"}


def _get_default_theme(panel_theme: str) -> str:
    return THEME_MAP.get(panel_theme, "default")


# See https://mermaid.js.org/schemas/config.schema.json
class MermaidConfiguration(pn.viewable.Viewer, WidgetBase):
    """\
    An interactive Widget for editing the Mermaid JS configuration.

    See https://mermaid.js.org/schemas/config.schema.json

    Example:
    ```python
    config = MermaidConfiguration()
    ```

    You can use it as a *reference value* for the MermaidDiagram:

    ```python
    diagram = MermaidDiagram(
        object="...",
        configuration=config,
        ...
    )
    """

    value: dict = param.Dict(
        constant=True,
        doc="""The mermaid configuration as described in https://mermaid.js.org/schemas/config.schema.json.""",
    )

    look: str = param.Selector(
        default="classic",
        objects=["classic", "handDrawn"],
        doc="""Defines which main look to use for the diagram.""",
    )
    theme: str = param.Selector(
        default="default",
        objects=["default", "base", "dark", "forest", "neutral"],
        doc="""Theme, the CSS style sheet.\nYou may also use `themeCSS` to override this value.\n""",
    )

    def __init__(self, **params):
        if "theme" not in params:
            params["theme"] = _get_default_theme(pn.config.theme)
        super().__init__(**params)

    @param.depends("look", "theme", watch=True, on_init=True)
    def _update_value(self):
        value = {"look": self.look, "theme": self.theme}
        value = {
            key: value
            for key, value in value.items()
            if value and value != self.param[key].default
        }

        with param.edit_constant(self):
            self.value = value

    def __panel__(self):
        return pn.Column(
            pn.pane.Markdown(
                """### Mermaid Configuration
            """,
                margin=(0, 5, 0, 5),
            ),
            self.param.look,
            self.param.theme,
        )


class MermaidDiagram(JSComponent):
    '''\
    An interactive `MermaidDiagram` pane based on [Mermaid JS](https://mermaid.js.org/).

    Example:

    ```python
    from panel_mermaid import MermaidDiagram

    pn.extension()

    MermaidDiagram(
        object=(
            """
            graph LR
                A[Hello] --- B[World]
            """
        )
    ).servable()
    ```

    If you want to use Font-Awesome icons prefixed with `fa:`, you
    can do this by including the Font-Awesome CSS in your application:

    ```python
    pn.extension(
        ...
        css_files=[
            "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.6.0/css/all.min.css"
        ],
    )
    ```
    '''

    object = param.String(
        default="",
        allow_refs=True,
        doc="""A mermaid diagram string like
            graph LR
                A --- B
                B-->C[forbidden]
                B-->D[allowed];""",
    )
    configuration = param.Dict(
        {},
        allow_refs=True,
        doc="""The mermaid configuration as described in https://mermaid.js.org/schemas/config.schema.json.""",
    )
    event: dict = param.Dict(
        allow_refs=False, doc="""An event from interacting with the diagram"""
    )
    event_configuration: list = param.List(
        item_type=tuple,
        doc="""List of (event name, query selector string) tuples to subscribe to.
        For example [("click", ".node")] or [("mouseover", ".node")].""",
        allow_refs=True,
    )

    _esm = "mermaid.esm.js"
    _styles = (
        "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.6.0/css/all.min.css"
    )

    _importmap = {
        "imports": {
            "mermaid": f"https://cdn.jsdelivr.net/npm/mermaid@{VERSION}/dist/mermaid.esm.min.mjs"
        }
    }



import textwrap

import panel as pn
from config import (
    ACCENT,
    BOOTSTRAP_CSS_URL,
    EXAMPLES,
    LOGO,
    get_code_editor_theme,
    get_json_theme,
)
from example_picker import ExamplePicker
from panel_modal import Modal
from splitter import Splitter

from panel_mermaid import MermaidConfiguration, MermaidDiagram

# Panel Configuration
pn.extension(
    "codeeditor", "modal", sizing_mode="stretch_width", css_files=[BOOTSTRAP_CSS_URL]
)

# Component Definitions
config = MermaidConfiguration()
example = ExamplePicker(value="Default", options=EXAMPLES)
code = pn.widgets.CodeEditor(
    value=example.value,
    theme=get_code_editor_theme(),
    sizing_mode="stretch_both",
    on_keyup=False,
)
diagram = MermaidDiagram(
    object=code.value,
    configuration=config,
    event_configuration=[("click", ".node")],
    margin=10,
)
run = pn.widgets.Button(
    name="▷ Run",
    sizing_mode="fixed",
    width=100,
    button_type="primary",
    description="Click Run to update the diagram",
    margin=(10, 10, 10, 20),
)
event_label = pn.pane.Markdown("### Mermaid Events", margin=(0, 5, 0, 5))
event = pn.pane.JSON(
    object=diagram.param.event, name="Config Dict", theme=get_json_theme()
)
events = pn.Column(
    event_label,
    diagram.param.event_configuration,
    event,
)

docs_modal = Modal()
docs_section = pn.Column(
    pn.pane.Markdown("### Mermaid Documentation", margin=(0, 5, 0, 5)),
    pn.widgets.Button.from_param(
        docs_modal.param.open,
        name="Show",
        button_type="primary",
        button_style="outline",
        description="Click here to show the MermaidDiagram documentation",
    ),
    docs_modal,
)


# Callback Definitions
@pn.depends(run, code, watch=True)
def _update_diagram(event=None, value=""):
    diagram.object = value


@pn.depends(example, watch=True)
def _update_code(value):
    code.value = value
    _update_diagram(value=value)


@pn.depends(docs_modal.param.open)
def docs_content(event=None):
    return pn.Column(
        pn.pane.Markdown(
            textwrap.dedent(diagram.__doc__) + "\n" + diagram.param._repr_html_(),
        ),
        height=500,
        scroll=True,
    )


# Layout Setup
docs_modal[:] = [docs_content]

sidebar = [
    example,
    config,
    events,
    docs_section,
]

main_content = pn.Column(
    run,
    pn.pane.Markdown(
        "Click *run* or press *CTRL+Enter* in the editor to update the diagram.",
        margin=(-20, 15, -25, 15),
    ),
    Splitter(
        left=code,
        right=diagram,
        sizing_mode="stretch_both",
        margin=(10, 10, 50, 10),
    ),
)

# Template Setup
pn.template.FastListTemplate(
    title="Panel Mermaid | Editor",
    logo=LOGO,
    sidebar=sidebar,
    main=[main_content],
    main_layout=None,
    accent=ACCENT,
).servable()



import panel as pn

from panel_mermaid import MermaidDiagram

pn.extension()

MermaidDiagram(
    object=(
        """
        graph LR
            A[Hello] --- B[World]
        """
    )
).servable()
